@@ -54,14 +54,14 @@
 #'     (numeric)
 #'     A non-negative scalar on how intrusive (disruptive) the reporter to the user. This multiplicative scalar applies to the _interval_ and _times_ parameters. (Default: `1.0`)\cr
 #'   
-#'     \describe{
-#'       \item{\option{progressr.intrusiveness.auditory}:}{(numeric) intrusiveness for auditory progress handlers (Default: `5.0`)}
-#'       \item{\option{progressr.intrusiveness.file}:}{(numeric) intrusiveness for file-based progress handlers (Default: `5.0`)}
-#'       \item{\option{progressr.intrusiveness.gui}:}{(numeric) intrusiveness for graphical-user-interface progress handlers (Default: `1.0`)}
-#'       \item{\option{progressr.intrusiveness.notifier}:}{(numeric) intrusiveness for progress handlers that creates notifications (Default: `10.0`)}
-#'       \item{\option{progressr.intrusiveness.terminal}:}{(numeric) intrusiveness for progress handlers that outputs to the terminal (Default: `1.0`)}
-#'       \item{\option{progressr.intrusiveness.debug}:}{(numeric) intrusiveness for "debug" progress handlers (Default: `0.0`)}
-#'     }
+#'    \describe{
+#'      \item{\option{progressr.intrusiveness.audio}:}{(numeric) intrusiveness for auditory progress handlers (Default: `5.0`)}
+#'      \item{\option{progressr.intrusiveness.file}:}{(numeric) intrusiveness for file-based progress handlers (Default: `5.0`)}
+#'      \item{\option{progressr.intrusiveness.gui}:}{(numeric) intrusiveness for graphical-user-interface progress handlers (Default: `1.0`)}
+#'      \item{\option{progressr.intrusiveness.notifier}:}{(numeric) intrusiveness for progress handlers that creates notifications (Default: `10.0`)}
+#'      \item{\option{progressr.intrusiveness.terminal}:}{(numeric) intrusiveness for progress handlers that outputs to the terminal (Default: `1.0`)}
+#'      \item{\option{progressr.intrusiveness.debug}:}{(numeric) intrusiveness for "debug" progress handlers (Default: `0.0`)}
+#'    }
 #'   }
 #' }
 #'
@@ -130,7 +130,7 @@
 #' progressr.interrupts
 #' progressr.interval
 #' progressr.intrusiveness
-#' progressr.intrusiveness.auditory
+#' progressr.intrusiveness.audio
 #' progressr.intrusiveness.debug
 #' progressr.intrusiveness.file
 #' progressr.intrusiveness.gui
@@ -139,7 +139,6 @@
 #' progressr.handlers
 #' progressr.times
 #'
-#' @keywords internal
 #' @name progressr.options
 NULL
 
@@ -245,16 +244,27 @@ update_package_options <- function(debug = FALSE) {
 
   ## make_progression_handler() arguments
   update_package_option("clear", mode = "logical", default = TRUE, debug = debug)
-  update_package_option("enable", mode = "logical", default = interactive(), debug = debug)
+
+  ## Special case: Support R_PROGRESSR_ENABLE=interactive
+  value <- Sys.getenv("R_PROGRESSR_ENABLE", "")
+  if (value == "interactive") {
+    options(progressr.enable = interactive())
+  } else {
+    update_package_option("enable", mode = "logical", default = interactive(), debug = debug)
+  }
+
   update_package_option("enable_after", mode = "numeric", default = 0.0, debug = debug)
   update_package_option("interval", mode = "numeric", default = 0.0, debug = debug)
   update_package_option("times", mode = "numeric", default = +Inf, debug = debug)
   update_package_option("interrupts", mode = "logical", default = TRUE, debug = debug)
-  update_package_option("interrupt.message", mode = "character", default = "interrupt detected", debug = debug)
 
   ## Life-cycle, e.g. deprecation an defunct
   update_package_option("lifecycle.progress", mode = "character", default = "deprecated", debug = debug)
 
   ## However, not used
   update_package_option("global.handler", mode = "logical", debug = debug)
+
+  ## For RStudio users
+  update_package_option("rstudio.patch", mode = "logical", debug = debug)
 }
+
